import sys
from PyQt5.QtWidgets import QApplication
from asa_hmi_data_agent.mainwindow import MainWindow

def run():
    app = QApplication([])
    window = MainWindow()
    window.show()
    sys.exit(app.exec_())

if __name__ == "__main__":
    run()



from PyQt5.QtCore import QThread
from PyQt5.QtCore import QObject
from PyQt5.QtCore import pyqtSignal

from asa_hmi_data_agent.cli_tools.commands import *

import zmq
import json

STATE_REC = 1
STATE_REP = 2

class AdtSocketReceiver(QThread):
    signalTermOpen  = pyqtSignal(int, str, int)
    signalTermClose = pyqtSignal(int)
    signalTermClear = pyqtSignal(int)
    signalLoaderStart = pyqtSignal(str, str)
    signalLoaderState = pyqtSignal()

    def __init__(self, context, socket):
        QThread.__init__(self)
        self.context = context
        self.socket = socket
        self.status = STATE_REC

    def run(self):
        i = 0
        while self.context.closed is False:
            if self.status is STATE_REP:
                continue
            elif self.status is STATE_REC:
                cmd = json.loads(self.socket.recv_json())
                if cmd['cmd'] == AdtCmd.LOADER:
                    self.doCmdLoader(cmd)
                elif cmd['cmd'] == AdtCmd.TERM:
                    self.doCmdTerm(cmd)
                self.status = STATE_REP

    def doCmdLoader(self, cmd):
        if cmd['subcmd'] == AdtSubCmdLoader.START:
            self.signalLoaderStart.emit(cmd['port'], cmd['hexfile'])
        elif cmd['subcmd'] == AdtSubCmdLoader.STATE:
            self.signalLoaderState.emit()

    def doCmdTerm(self, cmd):
        if cmd['subcmd'] == AdtSubCmdTerm.OPEN:
            self.signalTermOpen.emit(cmd['id'], cmd['port'], cmd['baudrate'])
        elif cmd['subcmd'] == AdtSubCmdTerm.CLOSE:
            self.signalTermClose.emit(cmd['id'])
        elif cmd['subcmd'] == AdtSubCmdTerm.CLEAR:
            self.signalTermClear.emit(cmd['id'])

class AdtSocketHandler(QObject):
    signalTermOpen  = pyqtSignal(int, str, int)
    signalTermClose = pyqtSignal(int)
    signalTermClear = pyqtSignal(int)
    signalLoaderStart = pyqtSignal(str, str)
    signalLoaderState = pyqtSignal()

    def __init__(self):
        super(AdtSocketHandler, self).__init__()
        self.context = zmq.Context()
        self.socket = self.context.socket(zmq.REP)
        self.socket.bind('tcp://*:8787')

        self.receiver = AdtSocketReceiver(self.context, self.socket)
        self.receiver.start()

        self.receiver.signalTermOpen[int, str, int].connect(self.signalTermOpen.emit)
        self.receiver.signalTermClose[int].connect(self.signalTermClose.emit)
        self.receiver.signalTermClear[int].connect(self.signalTermClear.emit)
        self.receiver.signalLoaderStart[str, str].connect(self.signalLoaderStart.emit)
        self.receiver.signalLoaderState.connect(self.signalLoaderState.emit)

    def sendRes(self, res):
        self.socket.send_json(json.dumps(res))
        self.receiver.status = STATE_REC



import numpy as np
from asa_hmi_data_agent.hmipac.type import *

def arToStr(data):
    res = ''
    t   = getTypeNum(data.dtype.name)
    data_list = data.tolist()

    res += getTypeStr(t) + ':' + '\n'
    res += align(2, data_list)
    return res

def stToStr(data):
    res = ''
    fs  = npDtypeToFs(data.dtype) # formatString
    res += fs + ': \n'
    for f in data.dtype.names:
        s = arToStr(data[f])
        s = '\n'.join('  ' + e for e in s.split('\n')[0:-1]) + '\n'
        # last item of s.split('\n') is '', so remove it
        res += s
    return res

def align(prespace, list_data):
    ls   = [str(e) for e in list_data] # list of str(data)
    maxL = max([len(s) for s in ls]) # max length of str(data)
    n    = len(list_data) # num of list_data
    res  = ''
    LINEMAX = 61 # each line string max length
    if maxL > LINEMAX:
        pass
    else:
        s = 0 # start index
        e = 0 # end index
        l = '' # each line string tmp
        q = LINEMAX // (maxL+2)
        # each data print include ',' and ' '
        e = s + q
        while e < n:
            l  = ''.join(' ' for i in range(prespace))
            l += ', '.join(('{:>'+'{:d}'.format(maxL)+'s}').format(s) for s in ls[s:e])
            l += ','
            res += l + '\n'
            s = e
            e = s + q
        l = '  ' + ', '.join(('{:>'+'{:d}'.format(maxL)+'s}').format(s) for s in ls[s:n])
        res += l + '\n'
        return res



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/mainwindow.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(500, 700)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        self.centralWidget = QtWidgets.QWidget(MainWindow)
        font = QtGui.QFont()
        font.setFamily("微軟正黑體")
        self.centralWidget.setFont(font)
        self.centralWidget.setObjectName("centralWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralWidget)
        self.verticalLayout.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget_main = QtWidgets.QTabWidget(self.centralWidget)
        self.tabWidget_main.setObjectName("tabWidget_main")
        self.tabHmi = QtWidgets.QWidget()
        self.tabHmi.setObjectName("tabHmi")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.tabHmi)
        self.horizontalLayout.setContentsMargins(11, 11, 11, 11)
        self.horizontalLayout.setSpacing(6)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.tabHmi_1 = QtWidgets.QWidget(self.tabHmi)
        self.tabHmi_1.setObjectName("tabHmi_1")
        self.horizontalLayout.addWidget(self.tabHmi_1)
        self.tabHmi_2 = QtWidgets.QWidget(self.tabHmi)
        self.tabHmi_2.setObjectName("tabHmi_2")
        self.horizontalLayout.addWidget(self.tabHmi_2)
        self.tabWidget_main.addTab(self.tabHmi, "")
        self.tabAsaProg = QtWidgets.QWidget()
        self.tabAsaProg.setObjectName("tabAsaProg")
        self.tabWidget_main.addTab(self.tabAsaProg, "")
        self.tabAvrdude = QtWidgets.QWidget()
        self.tabAvrdude.setObjectName("tabAvrdude")
        self.tabWidget_main.addTab(self.tabAvrdude, "")
        self.tabAdtSettings = QtWidgets.QWidget()
        self.tabAdtSettings.setObjectName("tabAdtSettings")
        self.tabWidget_main.addTab(self.tabAdtSettings, "")
        self.verticalLayout.addWidget(self.tabWidget_main)
        MainWindow.setCentralWidget(self.centralWidget)

        self.retranslateUi(MainWindow)
        self.tabWidget_main.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "ASA_HMI_Data_Agent"))
        self.tabWidget_main.setTabText(self.tabWidget_main.indexOf(self.tabHmi), _translate("MainWindow", "HMI"))
        self.tabWidget_main.setTabText(self.tabWidget_main.indexOf(self.tabAsaProg), _translate("MainWindow", "燒錄M128"))
        self.tabWidget_main.setTabText(self.tabWidget_main.indexOf(self.tabAvrdude), _translate("MainWindow", "STK500燒錄"))
        self.tabWidget_main.setTabText(self.tabWidget_main.indexOf(self.tabAdtSettings), _translate("MainWindow", "設定"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/hmi_save_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_HmiSaveDialog(object):
    def setupUi(self, HmiSaveDialog):
        HmiSaveDialog.setObjectName("HmiSaveDialog")
        HmiSaveDialog.resize(630, 350)
        font = QtGui.QFont()
        font.setFamily("微軟正黑體")
        HmiSaveDialog.setFont(font)
        HmiSaveDialog.setModal(True)
        self.gridLayout_2 = QtWidgets.QGridLayout(HmiSaveDialog)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.tabWidget = QtWidgets.QTabWidget(HmiSaveDialog)
        self.tabWidget.setObjectName("tabWidget")
        self.tab_txt = QtWidgets.QWidget()
        self.tab_txt.setObjectName("tab_txt")
        self.gridLayout = QtWidgets.QGridLayout(self.tab_txt)
        self.gridLayout.setObjectName("gridLayout")
        self.textEdit_txt = QtWidgets.QTextEdit(self.tab_txt)
        self.textEdit_txt.setObjectName("textEdit_txt")
        self.gridLayout.addWidget(self.textEdit_txt, 0, 0, 3, 1)
        self.pushButton_txtSave = QtWidgets.QPushButton(self.tab_txt)
        self.pushButton_txtSave.setObjectName("pushButton_txtSave")
        self.gridLayout.addWidget(self.pushButton_txtSave, 0, 1, 1, 1)
        self.pushButton_txtClose = QtWidgets.QPushButton(self.tab_txt)
        self.pushButton_txtClose.setObjectName("pushButton_txtClose")
        self.gridLayout.addWidget(self.pushButton_txtClose, 1, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 220, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 2, 1, 1, 1)
        self.tabWidget.addTab(self.tab_txt, "")
        self.tab_mat = QtWidgets.QWidget()
        self.tab_mat.setObjectName("tab_mat")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.tab_mat)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.pushButton_matSave = QtWidgets.QPushButton(self.tab_mat)
        self.pushButton_matSave.setObjectName("pushButton_matSave")
        self.gridLayout_3.addWidget(self.pushButton_matSave, 1, 1, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem1, 4, 1, 1, 1)
        self.pushButton_matClose = QtWidgets.QPushButton(self.tab_mat)
        self.pushButton_matClose.setObjectName("pushButton_matClose")
        self.gridLayout_3.addWidget(self.pushButton_matClose, 2, 1, 1, 1)
        self.tableWidget_mat = QtWidgets.QTableWidget(self.tab_mat)
        self.tableWidget_mat.setObjectName("tableWidget_mat")
        self.tableWidget_mat.setColumnCount(4)
        self.tableWidget_mat.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget_mat.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget_mat.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget_mat.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget_mat.setHorizontalHeaderItem(3, item)
        self.gridLayout_3.addWidget(self.tableWidget_mat, 1, 0, 4, 1)
        self.tabWidget.addTab(self.tab_mat, "")
        self.gridLayout_2.addWidget(self.tabWidget, 0, 0, 1, 1)

        self.retranslateUi(HmiSaveDialog)
        self.tabWidget.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(HmiSaveDialog)
        HmiSaveDialog.setTabOrder(self.tabWidget, self.tableWidget_mat)
        HmiSaveDialog.setTabOrder(self.tableWidget_mat, self.pushButton_matSave)
        HmiSaveDialog.setTabOrder(self.pushButton_matSave, self.pushButton_matClose)

    def retranslateUi(self, HmiSaveDialog):
        _translate = QtCore.QCoreApplication.translate
        HmiSaveDialog.setWindowTitle(_translate("HmiSaveDialog", "HmiSaveDialog"))
        self.pushButton_txtSave.setText(_translate("HmiSaveDialog", "儲存檔案"))
        self.pushButton_txtClose.setText(_translate("HmiSaveDialog", "關閉"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_txt), _translate("HmiSaveDialog", "txt檔案"))
        self.pushButton_matSave.setText(_translate("HmiSaveDialog", "儲存檔案"))
        self.pushButton_matClose.setText(_translate("HmiSaveDialog", "關閉"))
        item = self.tableWidget_mat.horizontalHeaderItem(0)
        item.setText(_translate("HmiSaveDialog", "變數名稱"))
        item = self.tableWidget_mat.horizontalHeaderItem(1)
        item.setText(_translate("HmiSaveDialog", "資料型態"))
        item = self.tableWidget_mat.horizontalHeaderItem(2)
        item.setText(_translate("HmiSaveDialog", "數量"))
        item = self.tableWidget_mat.horizontalHeaderItem(3)
        item.setText(_translate("HmiSaveDialog", "總資料大小(bytes)"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_mat), _translate("HmiSaveDialog", "mat檔案"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/bit_selector.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_BitSelector(object):
    def setupUi(self, BitSelector):
        BitSelector.setObjectName("BitSelector")
        BitSelector.resize(526, 372)
        BitSelector.setSizeGripEnabled(False)
        self.gridLayout = QtWidgets.QGridLayout(BitSelector)
        self.gridLayout.setObjectName("gridLayout")
        self.groupBox_fuseLow = QtWidgets.QGroupBox(BitSelector)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.groupBox_fuseLow.setFont(font)
        self.groupBox_fuseLow.setObjectName("groupBox_fuseLow")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox_fuseLow)
        self.verticalLayout.setObjectName("verticalLayout")
        self.checkBox_fuseL7 = QtWidgets.QCheckBox(self.groupBox_fuseLow)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseL7.setFont(font)
        self.checkBox_fuseL7.setObjectName("checkBox_fuseL7")
        self.verticalLayout.addWidget(self.checkBox_fuseL7)
        self.checkBox_fuseL6 = QtWidgets.QCheckBox(self.groupBox_fuseLow)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseL6.setFont(font)
        self.checkBox_fuseL6.setObjectName("checkBox_fuseL6")
        self.verticalLayout.addWidget(self.checkBox_fuseL6)
        self.checkBox_fuseL5 = QtWidgets.QCheckBox(self.groupBox_fuseLow)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseL5.setFont(font)
        self.checkBox_fuseL5.setObjectName("checkBox_fuseL5")
        self.verticalLayout.addWidget(self.checkBox_fuseL5)
        self.checkBox_fuseL4 = QtWidgets.QCheckBox(self.groupBox_fuseLow)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseL4.setFont(font)
        self.checkBox_fuseL4.setObjectName("checkBox_fuseL4")
        self.verticalLayout.addWidget(self.checkBox_fuseL4)
        self.checkBox_fuseL3 = QtWidgets.QCheckBox(self.groupBox_fuseLow)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseL3.setFont(font)
        self.checkBox_fuseL3.setObjectName("checkBox_fuseL3")
        self.verticalLayout.addWidget(self.checkBox_fuseL3)
        self.checkBox_fuseL2 = QtWidgets.QCheckBox(self.groupBox_fuseLow)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseL2.setFont(font)
        self.checkBox_fuseL2.setObjectName("checkBox_fuseL2")
        self.verticalLayout.addWidget(self.checkBox_fuseL2)
        self.checkBox_fuseL1 = QtWidgets.QCheckBox(self.groupBox_fuseLow)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseL1.setFont(font)
        self.checkBox_fuseL1.setObjectName("checkBox_fuseL1")
        self.verticalLayout.addWidget(self.checkBox_fuseL1)
        self.checkBox_fuseL0 = QtWidgets.QCheckBox(self.groupBox_fuseLow)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseL0.setFont(font)
        self.checkBox_fuseL0.setObjectName("checkBox_fuseL0")
        self.verticalLayout.addWidget(self.checkBox_fuseL0)
        self.gridLayout.addWidget(self.groupBox_fuseLow, 0, 0, 1, 1)
        self.groupBox_lock = QtWidgets.QGroupBox(BitSelector)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.groupBox_lock.setFont(font)
        self.groupBox_lock.setObjectName("groupBox_lock")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.groupBox_lock)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.checkBox_lock7 = QtWidgets.QCheckBox(self.groupBox_lock)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_lock7.setFont(font)
        self.checkBox_lock7.setObjectName("checkBox_lock7")
        self.verticalLayout_4.addWidget(self.checkBox_lock7)
        self.checkBox_lock6 = QtWidgets.QCheckBox(self.groupBox_lock)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_lock6.setFont(font)
        self.checkBox_lock6.setObjectName("checkBox_lock6")
        self.verticalLayout_4.addWidget(self.checkBox_lock6)
        self.checkBox_lock5 = QtWidgets.QCheckBox(self.groupBox_lock)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_lock5.setFont(font)
        self.checkBox_lock5.setObjectName("checkBox_lock5")
        self.verticalLayout_4.addWidget(self.checkBox_lock5)
        self.checkBox_lock4 = QtWidgets.QCheckBox(self.groupBox_lock)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_lock4.setFont(font)
        self.checkBox_lock4.setObjectName("checkBox_lock4")
        self.verticalLayout_4.addWidget(self.checkBox_lock4)
        self.checkBox_lock3 = QtWidgets.QCheckBox(self.groupBox_lock)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_lock3.setFont(font)
        self.checkBox_lock3.setObjectName("checkBox_lock3")
        self.verticalLayout_4.addWidget(self.checkBox_lock3)
        self.checkBox_lock2 = QtWidgets.QCheckBox(self.groupBox_lock)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_lock2.setFont(font)
        self.checkBox_lock2.setObjectName("checkBox_lock2")
        self.verticalLayout_4.addWidget(self.checkBox_lock2)
        self.checkBox_lock1 = QtWidgets.QCheckBox(self.groupBox_lock)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_lock1.setFont(font)
        self.checkBox_lock1.setObjectName("checkBox_lock1")
        self.verticalLayout_4.addWidget(self.checkBox_lock1)
        self.checkBox_lock0 = QtWidgets.QCheckBox(self.groupBox_lock)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_lock0.setFont(font)
        self.checkBox_lock0.setObjectName("checkBox_lock0")
        self.verticalLayout_4.addWidget(self.checkBox_lock0)
        self.gridLayout.addWidget(self.groupBox_lock, 0, 3, 1, 1)
        self.groupBox_fuseHigh = QtWidgets.QGroupBox(BitSelector)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.groupBox_fuseHigh.setFont(font)
        self.groupBox_fuseHigh.setObjectName("groupBox_fuseHigh")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox_fuseHigh)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.checkBox_fuseH7 = QtWidgets.QCheckBox(self.groupBox_fuseHigh)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseH7.setFont(font)
        self.checkBox_fuseH7.setObjectName("checkBox_fuseH7")
        self.verticalLayout_2.addWidget(self.checkBox_fuseH7)
        self.checkBox_fuseH6 = QtWidgets.QCheckBox(self.groupBox_fuseHigh)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseH6.setFont(font)
        self.checkBox_fuseH6.setObjectName("checkBox_fuseH6")
        self.verticalLayout_2.addWidget(self.checkBox_fuseH6)
        self.checkBox_fuseH5 = QtWidgets.QCheckBox(self.groupBox_fuseHigh)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseH5.setFont(font)
        self.checkBox_fuseH5.setObjectName("checkBox_fuseH5")
        self.verticalLayout_2.addWidget(self.checkBox_fuseH5)
        self.checkBox_fuseH4 = QtWidgets.QCheckBox(self.groupBox_fuseHigh)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseH4.setFont(font)
        self.checkBox_fuseH4.setObjectName("checkBox_fuseH4")
        self.verticalLayout_2.addWidget(self.checkBox_fuseH4)
        self.checkBox_fuseH3 = QtWidgets.QCheckBox(self.groupBox_fuseHigh)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseH3.setFont(font)
        self.checkBox_fuseH3.setObjectName("checkBox_fuseH3")
        self.verticalLayout_2.addWidget(self.checkBox_fuseH3)
        self.checkBox_fuseH2 = QtWidgets.QCheckBox(self.groupBox_fuseHigh)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseH2.setFont(font)
        self.checkBox_fuseH2.setObjectName("checkBox_fuseH2")
        self.verticalLayout_2.addWidget(self.checkBox_fuseH2)
        self.checkBox_fuseH1 = QtWidgets.QCheckBox(self.groupBox_fuseHigh)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseH1.setFont(font)
        self.checkBox_fuseH1.setObjectName("checkBox_fuseH1")
        self.verticalLayout_2.addWidget(self.checkBox_fuseH1)
        self.checkBox_fuseH0 = QtWidgets.QCheckBox(self.groupBox_fuseHigh)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseH0.setFont(font)
        self.checkBox_fuseH0.setObjectName("checkBox_fuseH0")
        self.verticalLayout_2.addWidget(self.checkBox_fuseH0)
        self.gridLayout.addWidget(self.groupBox_fuseHigh, 0, 1, 1, 1)
        self.lineEdit_fuseLow = QtWidgets.QLineEdit(BitSelector)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.lineEdit_fuseLow.setFont(font)
        self.lineEdit_fuseLow.setObjectName("lineEdit_fuseLow")
        self.gridLayout.addWidget(self.lineEdit_fuseLow, 1, 0, 1, 1)
        self.lineEdit_fuseHigh = QtWidgets.QLineEdit(BitSelector)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.lineEdit_fuseHigh.setFont(font)
        self.lineEdit_fuseHigh.setObjectName("lineEdit_fuseHigh")
        self.gridLayout.addWidget(self.lineEdit_fuseHigh, 1, 1, 1, 1)
        self.lineEdit_fuseExtra = QtWidgets.QLineEdit(BitSelector)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.lineEdit_fuseExtra.setFont(font)
        self.lineEdit_fuseExtra.setObjectName("lineEdit_fuseExtra")
        self.gridLayout.addWidget(self.lineEdit_fuseExtra, 1, 2, 1, 1)
        self.label = QtWidgets.QLabel(BitSelector)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 2, 0, 1, 3)
        self.lineEdit_lock = QtWidgets.QLineEdit(BitSelector)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.lineEdit_lock.setFont(font)
        self.lineEdit_lock.setObjectName("lineEdit_lock")
        self.gridLayout.addWidget(self.lineEdit_lock, 1, 3, 1, 1)
        self.pushButton_close = QtWidgets.QPushButton(BitSelector)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.pushButton_close.setFont(font)
        self.pushButton_close.setObjectName("pushButton_close")
        self.gridLayout.addWidget(self.pushButton_close, 2, 3, 1, 1)
        self.groupBox_fuseExtra = QtWidgets.QGroupBox(BitSelector)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.groupBox_fuseExtra.setFont(font)
        self.groupBox_fuseExtra.setObjectName("groupBox_fuseExtra")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_fuseExtra)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.checkBox_fuseE7 = QtWidgets.QCheckBox(self.groupBox_fuseExtra)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseE7.setFont(font)
        self.checkBox_fuseE7.setObjectName("checkBox_fuseE7")
        self.verticalLayout_3.addWidget(self.checkBox_fuseE7)
        self.checkBox_fuseE6 = QtWidgets.QCheckBox(self.groupBox_fuseExtra)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseE6.setFont(font)
        self.checkBox_fuseE6.setObjectName("checkBox_fuseE6")
        self.verticalLayout_3.addWidget(self.checkBox_fuseE6)
        self.checkBox_fuseE5 = QtWidgets.QCheckBox(self.groupBox_fuseExtra)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseE5.setFont(font)
        self.checkBox_fuseE5.setObjectName("checkBox_fuseE5")
        self.verticalLayout_3.addWidget(self.checkBox_fuseE5)
        self.checkBox_fuseE4 = QtWidgets.QCheckBox(self.groupBox_fuseExtra)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseE4.setFont(font)
        self.checkBox_fuseE4.setObjectName("checkBox_fuseE4")
        self.verticalLayout_3.addWidget(self.checkBox_fuseE4)
        self.checkBox_fuseE3 = QtWidgets.QCheckBox(self.groupBox_fuseExtra)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseE3.setFont(font)
        self.checkBox_fuseE3.setObjectName("checkBox_fuseE3")
        self.verticalLayout_3.addWidget(self.checkBox_fuseE3)
        self.checkBox_fuseE2 = QtWidgets.QCheckBox(self.groupBox_fuseExtra)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseE2.setFont(font)
        self.checkBox_fuseE2.setObjectName("checkBox_fuseE2")
        self.verticalLayout_3.addWidget(self.checkBox_fuseE2)
        self.checkBox_fuseE1 = QtWidgets.QCheckBox(self.groupBox_fuseExtra)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseE1.setFont(font)
        self.checkBox_fuseE1.setObjectName("checkBox_fuseE1")
        self.verticalLayout_3.addWidget(self.checkBox_fuseE1)
        self.checkBox_fuseE0 = QtWidgets.QCheckBox(self.groupBox_fuseExtra)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        font.setPointSize(12)
        self.checkBox_fuseE0.setFont(font)
        self.checkBox_fuseE0.setObjectName("checkBox_fuseE0")
        self.verticalLayout_3.addWidget(self.checkBox_fuseE0)
        self.gridLayout.addWidget(self.groupBox_fuseExtra, 0, 2, 1, 1)

        self.retranslateUi(BitSelector)
        QtCore.QMetaObject.connectSlotsByName(BitSelector)
        BitSelector.setTabOrder(self.checkBox_fuseL7, self.lineEdit_fuseLow)
        BitSelector.setTabOrder(self.lineEdit_fuseLow, self.checkBox_fuseH7)
        BitSelector.setTabOrder(self.checkBox_fuseH7, self.lineEdit_fuseHigh)
        BitSelector.setTabOrder(self.lineEdit_fuseHigh, self.checkBox_fuseE5)
        BitSelector.setTabOrder(self.checkBox_fuseE5, self.lineEdit_fuseExtra)
        BitSelector.setTabOrder(self.lineEdit_fuseExtra, self.checkBox_lock4)
        BitSelector.setTabOrder(self.checkBox_lock4, self.lineEdit_lock)
        BitSelector.setTabOrder(self.lineEdit_lock, self.pushButton_close)

    def retranslateUi(self, BitSelector):
        _translate = QtCore.QCoreApplication.translate
        BitSelector.setWindowTitle(_translate("BitSelector", "BitsSelector"))
        self.groupBox_fuseLow.setTitle(_translate("BitSelector", "Fuse Low"))
        self.checkBox_fuseL7.setText(_translate("BitSelector", "Bit7"))
        self.checkBox_fuseL6.setText(_translate("BitSelector", "Bit6"))
        self.checkBox_fuseL5.setText(_translate("BitSelector", "Bit5"))
        self.checkBox_fuseL4.setText(_translate("BitSelector", "Bit4"))
        self.checkBox_fuseL3.setText(_translate("BitSelector", "Bit3"))
        self.checkBox_fuseL2.setText(_translate("BitSelector", "Bit2"))
        self.checkBox_fuseL1.setText(_translate("BitSelector", "Bit1"))
        self.checkBox_fuseL0.setText(_translate("BitSelector", "Bit0"))
        self.groupBox_lock.setTitle(_translate("BitSelector", "Lock"))
        self.checkBox_lock7.setText(_translate("BitSelector", "Bit7"))
        self.checkBox_lock6.setText(_translate("BitSelector", "Bit6"))
        self.checkBox_lock5.setText(_translate("BitSelector", "Bit5"))
        self.checkBox_lock4.setText(_translate("BitSelector", "Bit4"))
        self.checkBox_lock3.setText(_translate("BitSelector", "Bit3"))
        self.checkBox_lock2.setText(_translate("BitSelector", "Bit2"))
        self.checkBox_lock1.setText(_translate("BitSelector", "Bit1"))
        self.checkBox_lock0.setText(_translate("BitSelector", "Bit0"))
        self.groupBox_fuseHigh.setTitle(_translate("BitSelector", "Fuse High"))
        self.checkBox_fuseH7.setText(_translate("BitSelector", "Bit7"))
        self.checkBox_fuseH6.setText(_translate("BitSelector", "Bit6"))
        self.checkBox_fuseH5.setText(_translate("BitSelector", "Bit5"))
        self.checkBox_fuseH4.setText(_translate("BitSelector", "Bit4"))
        self.checkBox_fuseH3.setText(_translate("BitSelector", "Bit3"))
        self.checkBox_fuseH2.setText(_translate("BitSelector", "Bit2"))
        self.checkBox_fuseH1.setText(_translate("BitSelector", "Bit1"))
        self.checkBox_fuseH0.setText(_translate("BitSelector", "Bit0"))
        self.label.setText(_translate("BitSelector", "Only support M128, M88 now!"))
        self.pushButton_close.setText(_translate("BitSelector", "Close"))
        self.groupBox_fuseExtra.setTitle(_translate("BitSelector", "Fuse Extra"))
        self.checkBox_fuseE7.setText(_translate("BitSelector", "Bit7"))
        self.checkBox_fuseE6.setText(_translate("BitSelector", "Bit6"))
        self.checkBox_fuseE5.setText(_translate("BitSelector", "Bit5"))
        self.checkBox_fuseE4.setText(_translate("BitSelector", "Bit4"))
        self.checkBox_fuseE3.setText(_translate("BitSelector", "Bit3"))
        self.checkBox_fuseE2.setText(_translate("BitSelector", "Bit2"))
        self.checkBox_fuseE1.setText(_translate("BitSelector", "Bit1"))
        self.checkBox_fuseE0.setText(_translate("BitSelector", "Bit0"))




from asa_hmi_data_agent.listport import getAvailableSerialPorts
from asa_hmi_data_agent.hmi.hmi_save_dialog import HmiSaveDialog
from asa_hmi_data_agent.hmi.hmi_load_dialog import HmiLoadDialog
from asa_hmi_data_agent.hmi.text_to_data import getFirstArray, getFirstStruct, textToData
from asa_hmi_data_agent.hmi.data_to_text import arToStr, stToStr
from asa_hmi_data_agent.ui.ui_hmi import Ui_MainWidgetHMI
from asa_hmi_data_agent import hmipac
import asa_hmi_data_agent.hmipac.type as tp

from PyQt5.QtCore import QObject, QThread, pyqtSlot, pyqtSignal
from PyQt5.QtWidgets import QFileDialog

import numpy as np
import datetime
import scipy.io
import serial
import sys
import os

# ---- class Serial Thread Start -----------------------------------------------
class SerialThread(QThread):
    sigGetLine       = pyqtSignal(str)
    sigGetArrayData  = pyqtSignal(np.ndarray)
    sigGetStructData = pyqtSignal(np.ndarray)
    sigLoseConnect   = pyqtSignal()
    sigFatalError    = pyqtSignal()
    
    def __init__(self, ser):
        QThread.__init__(self)
        self.ser = ser
        self.de = hmipac.Decoder()

    def run(self):
        while (self.ser.isOpen()):
            try:
                ch = self.ser.read(1)
            except serial.serialutil.SerialException as e:
                self.sigLoseConnect.emit()
                break
            else:
                self.de.add_text(ch)
                try:
                    type, data = self.de.get()
                except:
                    sigFatalError.emit()
                else:
                    if type is 0:
                        pass
                    elif type is 1:
                        self.sigGetArrayData.emit(data)
                    elif type is 2:
                        self.sigGetStructData.emit(data)
                    elif type is 3:
                        self.sigGetLine.emit(data)

# ---- class Serial Thread End -------------------------------------------------

class HMI(QObject):
    sigChangeWindowTitle = pyqtSignal(str)
    sigSerialPortCheck = pyqtSignal(bool, str)

    # ---- __init__ start ------------------------------------------------------
    def __init__(self, widget):
        super(HMI, self).__init__()
        self.ui = Ui_MainWidgetHMI()
        self.ui.setupUi(widget)
        self.hmiSaveDialog = HmiSaveDialog()
        self.hmiLoadDialog = HmiLoadDialog()

        # ---- Serial object Init Start ----------------------------------------
        self.ser = serial.Serial()
        self.ser.baudrate = 38400
        self.ser.timeout = 10
        # ---- Serial object Init End ------------------------------------------

        # ---- Serial Thread Init Start ----------------------------------------
        self.SerialThread = SerialThread(self.ser)
        self.SerialThread.sigGetLine[str].connect(self.text_terminalAppendLineFromDevice)
        self.SerialThread.sigGetArrayData[np.ndarray].connect(self.rec_AppendArray)
        self.SerialThread.sigGetStructData[np.ndarray].connect(self.rec_AppendStruct)
        self.SerialThread.sigLoseConnect.connect(self.loseConnectHandler)
        self.SerialThread.sigFatalError.connect(lambda:self.loseConnectHandler)
        # ---- Serial Thread Init End ------------------------------------------

        # ---- Function Linking start ------------------------------------------
        # 串列埠設定區
        self.s_updatePortlist()
        self.ui.s_btnPortToggle.clicked.connect(self.s_portToggle)
        self.ui.s_btnUpdatePortList.clicked.connect(self.s_updatePortlist)
        # 文字對話區
        self.ui.text_lineEditToBeSend.returnPressed.connect(self.text_sendLineToDevice)
        self.ui.text_btnSend.clicked.connect(self.text_sendLineToDevice)
        self.ui.text_btnClearTerminal.clicked.connect(self.text_terminalClear)
        self.ui.text_btnSaveTerminal.clicked.connect(self.text_terminalSave)
        # 接收區
        self.ui.rec_btnClear.clicked.connect(self.rec_textEditClear)
        self.ui.rec_btnMoveToSend.clicked.connect(self.rec_textEditMovetoSend)
        self.ui.rec_btnSave.clicked.connect(lambda : self.hmiSaveDialog.showAndLoadText(self.ui.rec_textEdit.toPlainText()))
        self.ui.rec_btnQuickSave.clicked.connect(lambda : self.quickSave(self.ui.rec_textEdit.toPlainText()))
        # 發送區
        self.ui.send_btnClear.clicked.connect(self.send_textEditClear)
        self.ui.send_btnReadFile.clicked.connect(self.hmiLoadDialog.show)
        self.ui.send_btnSendArray.clicked.connect(self.send_btnSendArray)
        self.ui.send_btnSendStruct.clicked.connect(self.send_btnSendStruct)
        self.ui.send_btnSave.clicked.connect(lambda : self.hmiSaveDialog.showAndLoadText(self.ui.send_textEdit.toPlainText()))
        self.ui.send_btnQuickSave.clicked.connect(lambda : self.quickSave(self.ui.send_textEdit.toPlainText()))
        # ---- Function Linking end --------------------------------------------

        # ---- HmiSaveDialog section start -------------------------------------
        self.hmiSaveDialog.accepted.connect(lambda : debugLog('HmiSaveDialog close'))
        # ---- HmiSaveDialog section end ---------------------------------------

        # ---- HmiLoadDialog section start -------------------------------------
        self.hmiLoadDialog.accepted.connect(self.updateTextFromLoadDialog)
        # ---- HmiLoadDialog section end ---------------------------------------

    # ---- 串列埠設定區功能實現 start --------------------------------------------
    # Update port list in s_portComboBox
    def s_updatePortlist(self):
        if self.ser.isOpen():
            pass
        else :
            availablePorts = getAvailableSerialPorts()
            debugLog('Update ports: {}'.format(str(availablePorts)))
            self.ui.s_portComboBox.clear()
            for port in availablePorts:
                self.ui.s_portComboBox.addItem(port)

    # Open/Close serial port
    def s_portToggle(self):
        if (self.ui.s_portComboBox.currentText() is '') and (self.ser.isOpen() is False) :
            pass
        elif (self.ser.isOpen() is False):
            self.ser.port = self.ui.s_portComboBox.currentText()
            debugLog('Try to open port: {}'.format(self.ser.port))
            try:
                self.ser.open()
                self.sigChangeWindowTitle.emit('ASA_HMI_Data_Agent  - {} is opened.'.format(self.ser.port))

            except serial.serialutil.SerialException as e:
                debugLog('Open port {} failed!'.format(self.ser.port))
                self.hmilog('Open port {} failed!'.format(self.ser.port))
            else :
                self.ui.s_btnPortToggle.setText("關閉串列埠")
                self.hmilog('Open {} success!'.format(self.ser.port))
                self.SerialThread.start()
        elif (self.ser.isOpen()):
            debugLog('Try to close port: {}'.format(self.ser.port))
            self.SerialThread.terminate()
            self.ser.close()
            self.sigChangeWindowTitle.emit('ASA_HMI_Data_Agent  - {} is closed.'.format(self.ser.port))
            self.ui.s_btnPortToggle.setText("開啟串列埠")
            debugLog('Close {} success!'.format(self.ser.port))
            self.hmilog('Close {} success!'.format(self.ser.port))
    # ---- 串列埠設定區功能實現 end ----------------------------------------------

    # ---- 文字對話區功能實現 start ----------------------------------------------
    # Append the line from serial in terminal
    def text_terminalAppendLineFromDevice(self, line):
        self.ui.text_terminal.append('>>  '+line)
        debugLog('Get  line: ' + line)

    # Send line to serial
    def text_sendLineToDevice(self):
        if self.ser.isOpen() is False:
            return False
        else:
            line = self.ui.text_lineEditToBeSend.text()
            self.ui.text_lineEditToBeSend.clear()
            self.ser.write(bytes(line+'\n', encoding = "utf-8") )
            self.ui.text_terminal.append('<<  '+line)
            debugLog('Send line: ' + line)

    # Svae the text in text terminal
    def text_terminalSave(self):
        name, _ = QFileDialog.getSaveFileName(
            filter='txt Files (*.txt);;All Files(*)',
            initialFilter='txt Files (*.txt)'
        )
        if name is '':
            return
        text = self.ui.text_terminal.toPlainText()
        with open(name, 'w') as f:
            f.write(text)

    # Clear the text in text terminal
    def text_terminalClear(self):
        text = self.ui.text_terminal.clear()

    def hmilog(self, s):
        self.ui.text_terminal.append('(log: {})'.format(s))
    # ---- 文字對話區功能實現 end ------------------------------------------------

    # ---- 接收區功能實現 start -------------------------------------------------
    def rec_textEditClear(self):
        self.ui.rec_textEdit.clear()

    def rec_textEditMovetoSend(self):
        text = self.ui.rec_textEdit.toPlainText()
        self.ui.rec_textEdit.clear()
        self.ui.send_textEdit.append(text)

    def rec_AppendArray(self, data):
        self.ui.rec_textEdit.append(arToStr(data))
        self.hmilog('Get  {} {} array data.'.format(
            str(data.size),
            tp.getTypeStr(tp.getTypeNum(data.dtype.name))
        ))
        debugLog('Get array: '+ str(data))

    def rec_AppendStruct(self, data):
        self.ui.rec_textEdit.append(stToStr(data))
        self.hmilog('Get  {} struct data.'.format(tp.getFs(data.dtype)))
        debugLog('Get struct: '+ str(data))

    # ---- 接收區功能實現 end ---------------------------------------------------

    # ---- 發送區功能實現 start -------------------------------------------------
    def send_textEditClear(self):
        self.ui.send_textEdit.clear()

    def send_verifyShowOK(self):
        self.ui.send_textBrowserVerify.clear()
        self.ui.send_textBrowserVerify.append('OK')

    def send_verifyShowFAIL(self):
        self.ui.send_textBrowserVerify.clear()
        self.ui.send_textBrowserVerify.append('FAIL')

    def send_btnSendArray(self):
        text = self.ui.send_textEdit.toPlainText()
        try:
            usedLines, data = getFirstArray(text)
        except (ValueError,SyntaxError,TypeError,UnboundLocalError) as e:
            debugLog('getFirstArray exception:' + str(type(e)))
            self.send_verifyShowFAIL()
        else:
            self.ser.write(hmipac.encodeArToPac(data))
            lines = text.split('\n')
            if usedLines == len(lines):
                text = ''
            else:
                text = '\n'.join(l for l in lines[usedLines::])
            self.ui.send_textEdit.clear()
            self.ui.send_textEdit.append(text)
            self.hmilog('Send {} {} array data.'.format(
                str(data.size),
                tp.getTypeStr(tp.getTypeNum(data.dtype.name))
            ))

    def send_btnSendStruct(self):
        text = self.ui.send_textEdit.toPlainText()
        try:
            usedLines, data = getFirstStruct(text)
        except (ValueError,SyntaxError,TypeError,UnboundLocalError) as e:
            debugLog('getFirstStruct exception:' + str(type(e)))
            self.send_verifyShowFAIL()
        else:
            self.ser.write(hmipac.encodeStToPac(data))
            lines = text.split('\n')
            text = '\n'.join(l for l in lines[usedLines::])
            self.ui.send_textEdit.clear()
            self.ui.send_textEdit.append(text)
            self.hmilog('Send {} struct data.'.format(tp.getFs(data.dtype)))

    # ---- 發送區功能實現 end ---------------------------------------------------

    def updateTextFromLoadDialog(self):
        if self.hmiLoadDialog.resText != '':
            self.ui.send_textEdit.append(self.hmiLoadDialog.resText)

    def loseConnectHandler(self):
        self.hmilog('log: Lost connect with '+self.ser.port+'!')
        self.ser.close()
        self.ui.s_btnPortToggle.setText("開啟串列埠")
        self.sigChangeWindowTitle.emit('ASA_HMI_Data_Agent  - Lost connect with {}!'.format(self.ser.port))
        self.s_updatePortlist()

    def fatalErrorHandler(self):
        self.hmilog('log: Fatal error 請確認M128程式正確，並重新開啟串列埠!')
        self.ser.close()
        self.ui.s_btnPortToggle.setText("開啟串列埠")
        self.sigChangeWindowTitle.emit('ASA_HMI_Data_Agent  - {} is closed.'.format(self.ser.port))
        self.s_updatePortlist()

    def quickSave(self, text):
        name, _ = QFileDialog.getSaveFileName(
            filter='Mat Files (*.mat);;txt Files (*.txt)',
            initialFilter='Mat Files (*.mat)'
        )
        if name is '':
            return

        file_extension = os.path.splitext(name)[-1]
        if file_extension == '.txt':
            with open(name, 'w') as f:
                f.write(text)
        elif file_extension == '.mat':
            try:
                datalist = textToData(text)
            except (ValueError,SyntaxError,TypeError,UnboundLocalError) as e:
                # TODO ERROR Dialog
                debugLog('quickSave exception: ' + str(type(e)) )
                pass
            else:
                d = dict()
                for i in range(len(datalist)):
                    d['data'+str(i)] = datalist[i]
                scipy.io.savemat(name, d)
        elif file_extension == '.csv':
            pass
        else:
            with open(name, 'w') as f:
                f.write(text)

def debugLog(msg):
    s = '[{}] terminal log: {}'.format(
        datetime.datetime.now().strftime("%H:%M:%S"),
        msg
    )
    print(s)



import inspect
import os
import sys

def get_script_dir(follow_symlinks=True):
    if getattr(sys, 'frozen', False): # py2exe, PyInstaller, cx_Freeze
        path = os.path.abspath(sys.executable)
    else:
        path = inspect.getabsfile(get_script_dir)
    if follow_symlinks:
        path = os.path.realpath(path)
    return os.path.dirname(path)

ADTPATH = get_script_dir()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/hmi.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWidgetHMI(object):
    def setupUi(self, MainWidgetHMI):
        MainWidgetHMI.setObjectName("MainWidgetHMI")
        MainWidgetHMI.resize(500, 700)
        font = QtGui.QFont()
        font.setFamily("微軟正黑體")
        MainWidgetHMI.setFont(font)
        self.topLayout = QtWidgets.QVBoxLayout(MainWidgetHMI)
        self.topLayout.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.topLayout.setContentsMargins(-1, 0, -1, 0)
        self.topLayout.setSpacing(6)
        self.topLayout.setObjectName("topLayout")
        self.groupBox_Serial = QtWidgets.QGroupBox(MainWidgetHMI)
        self.groupBox_Serial.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_Serial.sizePolicy().hasHeightForWidth())
        self.groupBox_Serial.setSizePolicy(sizePolicy)
        self.groupBox_Serial.setMinimumSize(QtCore.QSize(0, 0))
        self.groupBox_Serial.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.groupBox_Serial.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.groupBox_Serial.setObjectName("groupBox_Serial")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.groupBox_Serial)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label = QtWidgets.QLabel(self.groupBox_Serial)
        self.label.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.label.setBaseSize(QtCore.QSize(0, 0))
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        self.s_portComboBox = QtWidgets.QComboBox(self.groupBox_Serial)
        self.s_portComboBox.setObjectName("s_portComboBox")
        self.horizontalLayout_2.addWidget(self.s_portComboBox)
        self.s_btnUpdatePortList = QtWidgets.QPushButton(self.groupBox_Serial)
        self.s_btnUpdatePortList.setObjectName("s_btnUpdatePortList")
        self.horizontalLayout_2.addWidget(self.s_btnUpdatePortList)
        self.s_btnPortToggle = QtWidgets.QPushButton(self.groupBox_Serial)
        self.s_btnPortToggle.setObjectName("s_btnPortToggle")
        self.horizontalLayout_2.addWidget(self.s_btnPortToggle)
        self.topLayout.addWidget(self.groupBox_Serial)
        self.groupBox_Terminal = QtWidgets.QGroupBox(MainWidgetHMI)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_Terminal.sizePolicy().hasHeightForWidth())
        self.groupBox_Terminal.setSizePolicy(sizePolicy)
        self.groupBox_Terminal.setMinimumSize(QtCore.QSize(0, 150))
        self.groupBox_Terminal.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.groupBox_Terminal.setBaseSize(QtCore.QSize(0, 0))
        self.groupBox_Terminal.setObjectName("groupBox_Terminal")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox_Terminal)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.text_terminal = QtWidgets.QTextBrowser(self.groupBox_Terminal)
        self.text_terminal.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.text_terminal.sizePolicy().hasHeightForWidth())
        self.text_terminal.setSizePolicy(sizePolicy)
        self.text_terminal.setMinimumSize(QtCore.QSize(0, 60))
        font = QtGui.QFont()
        font.setFamily("Consolas")
        self.text_terminal.setFont(font)
        self.text_terminal.setObjectName("text_terminal")
        self.verticalLayout_2.addWidget(self.text_terminal)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.text_lineEditToBeSend = QtWidgets.QLineEdit(self.groupBox_Terminal)
        self.text_lineEditToBeSend.setEnabled(True)
        self.text_lineEditToBeSend.setMinimumSize(QtCore.QSize(0, 20))
        font = QtGui.QFont()
        font.setFamily("Consolas")
        self.text_lineEditToBeSend.setFont(font)
        self.text_lineEditToBeSend.setObjectName("text_lineEditToBeSend")
        self.horizontalLayout.addWidget(self.text_lineEditToBeSend)
        self.text_btnSend = QtWidgets.QPushButton(self.groupBox_Terminal)
        self.text_btnSend.setObjectName("text_btnSend")
        self.horizontalLayout.addWidget(self.text_btnSend)
        self.text_btnClearTerminal = QtWidgets.QPushButton(self.groupBox_Terminal)
        self.text_btnClearTerminal.setObjectName("text_btnClearTerminal")
        self.horizontalLayout.addWidget(self.text_btnClearTerminal)
        self.text_btnSaveTerminal = QtWidgets.QPushButton(self.groupBox_Terminal)
        self.text_btnSaveTerminal.setObjectName("text_btnSaveTerminal")
        self.horizontalLayout.addWidget(self.text_btnSaveTerminal)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.topLayout.addWidget(self.groupBox_Terminal)
        self.groupBox_SendRead = QtWidgets.QGroupBox(MainWidgetHMI)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_SendRead.sizePolicy().hasHeightForWidth())
        self.groupBox_SendRead.setSizePolicy(sizePolicy)
        self.groupBox_SendRead.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.groupBox_SendRead.setObjectName("groupBox_SendRead")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_SendRead)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.tabWidget = QtWidgets.QTabWidget(self.groupBox_SendRead)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tabWidget.sizePolicy().hasHeightForWidth())
        self.tabWidget.setSizePolicy(sizePolicy)
        self.tabWidget.setObjectName("tabWidget")
        self.tabSend = QtWidgets.QWidget()
        self.tabSend.setObjectName("tabSend")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.tabSend)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout()
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.send_textEdit = QtWidgets.QTextEdit(self.tabSend)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        self.send_textEdit.setFont(font)
        self.send_textEdit.setObjectName("send_textEdit")
        self.verticalLayout_7.addWidget(self.send_textEdit)
        self.horizontalLayout_4.addLayout(self.verticalLayout_7)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.send_btnClear = QtWidgets.QPushButton(self.tabSend)
        self.send_btnClear.setObjectName("send_btnClear")
        self.verticalLayout_5.addWidget(self.send_btnClear)
        self.send_btnSendStruct = QtWidgets.QPushButton(self.tabSend)
        self.send_btnSendStruct.setObjectName("send_btnSendStruct")
        self.verticalLayout_5.addWidget(self.send_btnSendStruct)
        self.send_btnSendArray = QtWidgets.QPushButton(self.tabSend)
        self.send_btnSendArray.setObjectName("send_btnSendArray")
        self.verticalLayout_5.addWidget(self.send_btnSendArray)
        spacerItem = QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_5.addItem(spacerItem)
        self.send_btnQuickSave = QtWidgets.QPushButton(self.tabSend)
        self.send_btnQuickSave.setObjectName("send_btnQuickSave")
        self.verticalLayout_5.addWidget(self.send_btnQuickSave)
        self.send_btnSave = QtWidgets.QPushButton(self.tabSend)
        self.send_btnSave.setObjectName("send_btnSave")
        self.verticalLayout_5.addWidget(self.send_btnSave)
        self.send_btnReadFile = QtWidgets.QPushButton(self.tabSend)
        self.send_btnReadFile.setObjectName("send_btnReadFile")
        self.verticalLayout_5.addWidget(self.send_btnReadFile)
        spacerItem1 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout_5.addItem(spacerItem1)
        self.labelSendVerify = QtWidgets.QLabel(self.tabSend)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelSendVerify.sizePolicy().hasHeightForWidth())
        self.labelSendVerify.setSizePolicy(sizePolicy)
        self.labelSendVerify.setMinimumSize(QtCore.QSize(75, 23))
        self.labelSendVerify.setMaximumSize(QtCore.QSize(75, 23))
        self.labelSendVerify.setObjectName("labelSendVerify")
        self.verticalLayout_5.addWidget(self.labelSendVerify)
        self.send_textBrowserVerify = QtWidgets.QTextBrowser(self.tabSend)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.send_textBrowserVerify.sizePolicy().hasHeightForWidth())
        self.send_textBrowserVerify.setSizePolicy(sizePolicy)
        self.send_textBrowserVerify.setMinimumSize(QtCore.QSize(75, 25))
        self.send_textBrowserVerify.setMaximumSize(QtCore.QSize(75, 25))
        self.send_textBrowserVerify.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.send_textBrowserVerify.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.send_textBrowserVerify.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustToContentsOnFirstShow)
        self.send_textBrowserVerify.setObjectName("send_textBrowserVerify")
        self.verticalLayout_5.addWidget(self.send_textBrowserVerify)
        self.horizontalLayout_4.addLayout(self.verticalLayout_5)
        self.tabWidget.addTab(self.tabSend, "")
        self.tabReceive = QtWidgets.QWidget()
        self.tabReceive.setObjectName("tabReceive")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.tabReceive)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout()
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.rec_textEdit = QtWidgets.QTextEdit(self.tabReceive)
        font = QtGui.QFont()
        font.setFamily("Consolas")
        self.rec_textEdit.setFont(font)
        self.rec_textEdit.setObjectName("rec_textEdit")
        self.verticalLayout_8.addWidget(self.rec_textEdit)
        self.horizontalLayout_3.addLayout(self.verticalLayout_8)
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.rec_btnClear = QtWidgets.QPushButton(self.tabReceive)
        self.rec_btnClear.setObjectName("rec_btnClear")
        self.verticalLayout_6.addWidget(self.rec_btnClear)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_6.addItem(spacerItem2)
        self.rec_btnQuickSave = QtWidgets.QPushButton(self.tabReceive)
        self.rec_btnQuickSave.setObjectName("rec_btnQuickSave")
        self.verticalLayout_6.addWidget(self.rec_btnQuickSave)
        self.rec_btnSave = QtWidgets.QPushButton(self.tabReceive)
        self.rec_btnSave.setObjectName("rec_btnSave")
        self.verticalLayout_6.addWidget(self.rec_btnSave)
        self.rec_btnMoveToSend = QtWidgets.QPushButton(self.tabReceive)
        self.rec_btnMoveToSend.setObjectName("rec_btnMoveToSend")
        self.verticalLayout_6.addWidget(self.rec_btnMoveToSend)
        self.horizontalLayout_3.addLayout(self.verticalLayout_6)
        self.tabWidget.addTab(self.tabReceive, "")
        self.verticalLayout_3.addWidget(self.tabWidget)
        self.topLayout.addWidget(self.groupBox_SendRead)

        self.retranslateUi(MainWidgetHMI)
        self.tabWidget.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(MainWidgetHMI)
        MainWidgetHMI.setTabOrder(self.s_portComboBox, self.s_btnUpdatePortList)
        MainWidgetHMI.setTabOrder(self.s_btnUpdatePortList, self.s_btnPortToggle)
        MainWidgetHMI.setTabOrder(self.s_btnPortToggle, self.text_terminal)
        MainWidgetHMI.setTabOrder(self.text_terminal, self.text_lineEditToBeSend)
        MainWidgetHMI.setTabOrder(self.text_lineEditToBeSend, self.text_btnSend)
        MainWidgetHMI.setTabOrder(self.text_btnSend, self.text_btnClearTerminal)
        MainWidgetHMI.setTabOrder(self.text_btnClearTerminal, self.text_btnSaveTerminal)
        MainWidgetHMI.setTabOrder(self.text_btnSaveTerminal, self.tabWidget)
        MainWidgetHMI.setTabOrder(self.tabWidget, self.send_textEdit)
        MainWidgetHMI.setTabOrder(self.send_textEdit, self.send_btnClear)
        MainWidgetHMI.setTabOrder(self.send_btnClear, self.send_btnSendStruct)
        MainWidgetHMI.setTabOrder(self.send_btnSendStruct, self.send_btnSendArray)
        MainWidgetHMI.setTabOrder(self.send_btnSendArray, self.send_btnQuickSave)
        MainWidgetHMI.setTabOrder(self.send_btnQuickSave, self.send_btnSave)
        MainWidgetHMI.setTabOrder(self.send_btnSave, self.send_btnReadFile)
        MainWidgetHMI.setTabOrder(self.send_btnReadFile, self.send_textBrowserVerify)
        MainWidgetHMI.setTabOrder(self.send_textBrowserVerify, self.rec_textEdit)
        MainWidgetHMI.setTabOrder(self.rec_textEdit, self.rec_btnClear)
        MainWidgetHMI.setTabOrder(self.rec_btnClear, self.rec_btnQuickSave)
        MainWidgetHMI.setTabOrder(self.rec_btnQuickSave, self.rec_btnSave)
        MainWidgetHMI.setTabOrder(self.rec_btnSave, self.rec_btnMoveToSend)

    def retranslateUi(self, MainWidgetHMI):
        _translate = QtCore.QCoreApplication.translate
        self.groupBox_Serial.setTitle(_translate("MainWidgetHMI", "串列埠設定"))
        self.label.setText(_translate("MainWidgetHMI", "選擇串列埠："))
        self.s_btnUpdatePortList.setText(_translate("MainWidgetHMI", "更新串列埠"))
        self.s_btnPortToggle.setText(_translate("MainWidgetHMI", "開啟串列埠"))
        self.groupBox_Terminal.setTitle(_translate("MainWidgetHMI", "文字對話區"))
        self.text_btnSend.setText(_translate("MainWidgetHMI", "Send"))
        self.text_btnClearTerminal.setText(_translate("MainWidgetHMI", "清除對話框"))
        self.text_btnSaveTerminal.setText(_translate("MainWidgetHMI", "儲存對話框"))
        self.groupBox_SendRead.setTitle(_translate("MainWidgetHMI", "資料送收區"))
        self.send_btnClear.setText(_translate("MainWidgetHMI", "清除暫存區"))
        self.send_btnSendStruct.setText(_translate("MainWidgetHMI", "傳送結構形式"))
        self.send_btnSendArray.setText(_translate("MainWidgetHMI", "傳送矩陣形式"))
        self.send_btnQuickSave.setText(_translate("MainWidgetHMI", "儲存檔案"))
        self.send_btnSave.setText(_translate("MainWidgetHMI", "編輯並儲存"))
        self.send_btnReadFile.setText(_translate("MainWidgetHMI", "讀取檔案"))
        self.labelSendVerify.setText(_translate("MainWidgetHMI", "資料格式驗證"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabSend), _translate("MainWidgetHMI", "發送"))
        self.rec_btnClear.setText(_translate("MainWidgetHMI", "清除暫存區"))
        self.rec_btnQuickSave.setText(_translate("MainWidgetHMI", "儲存檔案"))
        self.rec_btnSave.setText(_translate("MainWidgetHMI", "修改並儲存"))
        self.rec_btnMoveToSend.setText(_translate("MainWidgetHMI", "移動到發送區"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabReceive), _translate("MainWidgetHMI", "接收"))




import os
import shutil
from setuptools import setup, find_packages

REQUIREMENTS = [
    'py-asa-loader',
    'numpy',
    'pypiwin32',
    'PyQt5',
    'pyserial',
    'pyzmq',
    'scipy',
    'sip'
]

def run():
    setup(
        name='asa-hmi-data-agent',
        version='0.5.4',
        description = 'ASA developing tools on PC.',
        long_description='',
        author = 'mickey9910326',
        author_email = 'mickey9910326@gmail.com',
        url='https://github.com/mickey9910326/ASA_HMI_Data_Agent',
        license = 'GPL v3',
        packages=find_packages(),
        package_data={ 'asa_hmi_data_agent': [ 'tools/*', 'settings/*', 'tmp/*', 'ui/*']},
        zip_safe=False,
        entry_points = {
            'console_scripts': [
                'asa_hmi_data_agent = asa_hmi_data_agent.__main__:run',
                'adt = asa_hmi_data_agent.__main__:run',
                'adt-term = asa_hmi_data_agent.cli_tools.tool_term:run',
                'adt-loader = asa_hmi_data_agent.cli_tools.tool_loader:run'
            ],
        },
        install_requires=REQUIREMENTS
    )

if __name__ == '__main__':
    run()



from asa_hmi_data_agent.util import ADTPATH

class AvrdudeConfParser():

    def __init__(self):
        self.confFile = ADTPATH+'/tools/avrdude.conf'

    def setConfFile(self, s):
        self.confFile = s

    def GetDeviceInfo(self):
        file = open(self.confFile,"r")
        status = 1
        devices = list()
        while 1:
            line = file.readline()
            if line is '':
                break
            elif line[0] is '#':
                continue

            if status is 1:
                if line.find('part') is 0:
                    status = 2
            elif status is 2: # id
                idx = line.find('id')
                if idx is 4 or idx is 5:
                    id =  line.split('\"')[1]
                    status = 3
            elif status is 3: # desc
                idx = line.find('desc')
                if idx is 4 or idx is 5:
                    desc = line.split('\"')[1]
                    if len(desc) > 18:
                        status = 1
                        # remove desc like 'AVR XMEGA family common values'
                    else:
                        status = 4
            elif status is 4: # signature
                idx = line.find('signature')
                if idx is 4 or idx is 5:
                    signature = line[line.find('=')+2:line.find(';')].split(' ')
                    devices.append({
                        'desc' : desc,
                        'id' : id,
                        'signature' : signature
                    })
                    status = 1
        file.close()
        return devices



from asa_hmi_data_agent.ui.ui_mainwindow import Ui_MainWindow

from asa_hmi_data_agent.adt_settings.adt_settings import AdtSettings
from asa_hmi_data_agent.listport import getAvailableSerialPorts
from asa_hmi_data_agent.socket_api import AdtSocketHandler
from asa_hmi_data_agent.asaloader import AsaLoader
from asa_hmi_data_agent.avrdude import Avrdude
from asa_hmi_data_agent.hmi import HMI

from PyQt5.QtWidgets import QMainWindow, QApplication, QFileDialog
from PyQt5.QtCore import pyqtSlot, QThread, pyqtSignal

import serial
import time
import sys

class MainWindow(QMainWindow, Ui_MainWindow):
    hmiSertIsTerminated = bool(False)

    # ---- __init__ start ------------------------------------------------------
    def __init__(self, parent=None):
        # ---- init ui start ---------------------------------------------------
        super(MainWindow, self).__init__(parent)
        self.setupUi(self)
        # ---- init ui end -----------------------------------------------------
        self.HMI = HMI(self.tabHmi_1)
        self.HMI.sigChangeWindowTitle.connect(
            lambda s: self.setWindowTitle(s)
        )

        self.HMI2 = HMI(self.tabHmi_2)
        self.tabHmi_2.hide()

        self.avrdude = Avrdude(self.tabAvrdude)
        self.avrdude.sigSerialPortCheck[bool, str].connect(self.serToggleHandler)

        self.asaLoader = AsaLoader(self.tabAsaProg)
        self.asaLoader.sigSerialPortCheck[bool, str].connect(self.serToggleHandler)

        self.adtSettings = AdtSettings()
        self.adtSettings.setupUi(self.tabAdtSettings)
        self.adtSettings.init()
        self.adtSettings.signalTermNumApply.connect(self.ctlHmiTermNum)

        self.adtSocketHandler = AdtSocketHandler()
        self.adtSocketHandler.signalTermOpen[int, str, int].connect(self.ctlHmiTermOpen)
        self.adtSocketHandler.signalTermClose[int].connect(self.ctlHmiTermClose)
        self.adtSocketHandler.signalTermClear[int].connect(self.ctlHmiTermClear)
        self.adtSocketHandler.signalLoaderStart[str, str].connect(self.ctlLoaderStart)
        self.adtSocketHandler.signalLoaderState.connect(self.ctlLoaderState)

    # --------------------------------------------------------------------------
    def serToggleHandler(self, b, port):
        if(
            b
            and self.HMI.ser.isOpen()
            and self.HMI.ser.port == port
        ):
            self.HMI.s_portToggle()
            self.hmiSertIsTerminated = True
        elif(
            b is False
            and self.hmiSertIsTerminated
            and not self.HMI.ser.isOpen()
            and self.HMI.ser.port == port
        ):
            self.HMI.s_portToggle()
            self.hmiSertIsTerminated = False

    def ctlHmiTermNum(self, num):
        if num is 1:
            self.tabHmi_2.hide()
        elif num is 2:
            self.tabHmi_2.show()

    def ctlHmiTermOpen(self, id, port, baudrate):
        if id is 1:
            target = self.HMI
            isHidden = self.tabHmi_1.isHidden()
        elif id is 2:
            target = self.HMI2
            isHidden = self.tabHmi_2.isHidden()

        if isHidden:
            err = True
            msg = 'Terminal {} is not available now, plz open it at adt tab settings'.format(id)
        elif target.ser.isOpen():
            err = True
            msg = 'Terminal {} has been opened.'.format(id)
        else:
            availablePorts = getAvailableSerialPorts()
            if port in availablePorts:
                err = False
                msg = ''
                target.ui.s_portComboBox.clear()
                for p in availablePorts:
                    target.ui.s_portComboBox.addItem(p)
                target.ui.s_portComboBox.setCurrentIndex(availablePorts.index(port))
                target.ser.baudrate = baudrate
                target.s_portToggle()
            else:
                err = True
                msg = 'port {} is not available.'.format(port)
        res = { 'err' : err, 'msg' : msg }
        self.adtSocketHandler.sendRes(res)

    def ctlHmiTermClose(self, id):
        if id is 1:
            target = self.HMI
            isHidden = self.tabHmi_1.isHidden()
        elif id is 2:
            target = self.HMI2
            isHidden = self.tabHmi_2.isHidden()

        if isHidden:
            err = True
            msg = 'Terminal {} is not available now, plz open it at adt tab settings'.format(id)
        elif target.ser.isOpen() is False:
            err = True
            msg = 'Terminal {} has been closed.'.format(id)
        else:
            err = False
            msg = ''
            target.s_portToggle()
        res = { 'err' : err, 'msg' : msg }
        self.adtSocketHandler.sendRes(res)

    def ctlHmiTermClear(self, id):
        if id is 1:
            target = self.HMI
            isHidden = self.tabHmi_1.isHidden()
        elif id is 2:
            target = self.HMI2
            isHidden = self.tabHmi_2.isHidden()

        if isHidden:
            err = True
            msg = 'Terminal {} is not available now, plz open it at adt tab settings'.format(id)
        else:
            err = False
            msg = ''
            target.text_terminalClear()
        res = { 'err' : err, 'msg' : msg }
        self.adtSocketHandler.sendRes(res)

    # --------------------------------------------------------------------------
    def ctlLoaderStart(self, port, hexfile):
        self.asaLoader.ui.lineEdit_selectFile.setText(hexfile)
        availablePorts = getAvailableSerialPorts()
        if port in availablePorts:
            self.asaLoader.ui.comboBox_selectPort.clear()
            for p in availablePorts:
                self.asaLoader.ui.comboBox_selectPort.addItem(p)
            self.asaLoader.ui.comboBox_selectPort.setCurrentIndex(availablePorts.index(port))
            self.asaLoader.startLoad()
            time.sleep(0.5) # wait loader get total_steps done
            err = False
            msg = ''
            max = self.asaLoader.loaderThread.loader.total_steps
        else:
            err = True
            msg = 'port {} is not available.'.format(port)
            max = 0
        res = {
            'err': err,
            'msg': msg,
            'max': max
        }
        self.adtSocketHandler.sendRes(res)

    def ctlLoaderState(self):
        res = {
            'err': False,
            'msg': '',
            'times': self.asaLoader.loaderThread.loader.times
        }
        self.adtSocketHandler.sendRes(res)
